#!/usr/bin/env python
# encoding: utf-8
r"""
Riemann solvers for the Euler equations

This module contains Riemann solvers for the Euler equations which have the
form (in 1d):

.. math::
    q_t + f(q)_x = 0

where

.. math::
    q(x,t) = \left [ \begin{array}{c} \rho \\ \rho u \\ E \end{array} \right ],

the flux function is

.. math::
    f(q) = \left [ \begin{array}{c} \rho u \\ \rho u^2 + p \\ u(E+p) \end{array}\right ].

and :math:`\rho` is the density, :math:`u` the velocity, :math:`E` is the
energy and :math:`p` is the pressure.

Unless otherwise noted, the ideal gas equation of state is used:

.. math::
    p = (\gamma - 1) \left (E - \frac{1}{2}\rho u^2 \right)
"""

import numpy as np

num_eqn = 3

def euler_roe_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""
    Roe Euler solver in 1d

    *aug_global* should contain -
     - *gamma* - (float) Ratio of the heat capacities
     - *gamma1* - (float) :math:`1 - \gamma`
     - *efix* - (bool) Whether to use an entropy fix or not

    See :ref:`pyclaw_rp` for more details.

    :Version: 1.0 (2009-6-26)
    """

    # Problem dimensions
    num_rp = q_l.shape[1]
    num_waves = 3

    # Return values
    wave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )

    # Solver parameters
    gamma1 = problem_data['gamma1']

    # Calculate Roe averages
    u, a, enthalpy = roe_averages(q_l,q_r,problem_data)[0:3]

    # Find eigenvector coefficients
    delta = q_r - q_l
    a2 = gamma1 / a**2 * ((enthalpy -u**2)*delta[0,...] + u*delta[1,...] - delta[2,...])
    a3 = (delta[1,...] + (a-u) * delta[0,...] - a*a2) / (2.0*a)
    a1 = delta[0,...] - a2 - a3

    # Compute the waves
    wave[0,0,...] = a1
    wave[1,0,...] = a1 * (u-a)
    wave[2,0,...] = a1 * (enthalpy - u*a)
    s[0,...] = u - a

    wave[0,1,...] = a2
    wave[1,1,...] = a2 * u
    wave[2,1,...] = a2 * 0.5 * u**2
    s[1,...] = u

    wave[0,2,...] = a3
    wave[1,2,...] = a3 * (u+a)
    wave[2,2,...] = a3 * (enthalpy + u*a)
    s[2,...] = u + a

    # Entropy fix
    if problem_data['efix']:
        raise NotImplementedError("Entropy fix has not been implemented!")
    else:
        # Godunov update
        s_index = np.zeros((2,num_rp))
        for m in range(num_eqn):
            for mw in range(num_waves):
                s_index[0,:] = s[mw,:]
                amdq[m,:] += np.min(s_index,axis=0) * wave[m,mw,:]
                apdq[m,:] += np.max(s_index,axis=0) * wave[m,mw,:]

    return wave,s,amdq,apdq

def euler_hll_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""
    HLL euler solver ::

        W_1 = Q_hat - Q_l    s_1 = min(u_l-c_l,u_l+c_l,lambda_roe_1,lambda_roe_2)
        W_2 = Q_r - Q_hat    s_2 = max(u_r-c_r,u_r+c_r,lambda_roe_1,lambda_roe_2)

        Q_hat = ( f(q_r) - f(q_l) - s_2 * q_r + s_1 * q_l ) / (s_1 - s_2)

    *problem_data* should contain:
     - *gamma* - (float) Ratio of the heat capacities
     - *gamma1* - (float) :math:`1 - \gamma`

    :Version: 1.0 (2014-03-04)
    """

    # Problem dimensions
    num_rp = q_l.shape[1]
    num_waves = 2

    # Return values
    wave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )

    # Solver parameters
    gamma1 = problem_data['gamma1']

    # Calculate Roe averages, right and left speeds
    u, a, _, pl, pr = roe_averages(q_l,q_r,problem_data)
    H_r = (q_r[2,:] + pr) / q_r[0,:]
    H_l = (q_l[2,:] + pl) / q_l[0,:]
    u_r = q_r[1,:] / q_r[0,:]
    u_l = q_l[1,:] / q_l[0,:]
    a_r = np.sqrt(gamma1 * (H_r - 0.5 * u_r**2))
    a_l = np.sqrt(gamma1 * (H_l - 0.5 * u_l**2))

    # Compute Einfeldt speeds
    s_index = np.empty((4,num_rp))
    s_index[0,:] = u + a
    s_index[1,:] = u - a
    s_index[2,:] = u_l + a_l
    s_index[3,:] = u_l - a_l
    s[0,:]  = np.min(s_index,axis=0)
    s_index[2,:] = u_r + a_r
    s_index[3,:] = u_r - a_r
    s[1,:] = np.max(s_index,axis=0)

    # Compute middle state
    q_hat = np.empty((num_eqn,num_rp))
    q_hat[0,:] = (q_r[1,:] - q_l[1,:] -
                    s[1,:] * q_r[0,:] + s[0,:] * q_l[0,:]) / (s[0,:] - s[1,:])
    q_hat[1,:] = (q_r[1,:]**2/q_r[0,:] + pr - (q_l[1,:]**2/q_l[0,:] + pl) -
                    s[1,:] * q_r[1,:] + s[0,:] * q_l[1,:]) / (s[0,:] - s[1,:])
    q_hat[2,:] = ((q_r[2,:] + pr)*q_r[1,:]/q_r[0,:] - (q_l[2,:] + pl)*q_l[1,:]/q_l[0,:] -
                    s[1,:] * q_r[2,:] + s[0,:] * q_l[2,:]) / (s[0,:] - s[1,:])

    # Compute each family of waves
    wave[:,0,:] = q_hat - q_l
    wave[:,1,:] = q_r - q_hat

    # Compute variations
    s_index = np.zeros((2,num_rp))
    for m in range(num_eqn):
        for mw in range(num_waves):
            s_index[0,:] = s[mw,:]
            amdq[m,:] += np.min(s_index,axis=0) * wave[m,mw,:]
            apdq[m,:] += np.max(s_index,axis=0) * wave[m,mw,:]

    return wave, s, amdq, apdq

def euler_hllc_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""
    HLLC Euler solver ::

        W_1 = q_hat_l - q_l      s_1 = min(u_l-c_l,u_l+c_l,lambda_roe_1,lambda_roe_2)
        W_2 = q_hat_r - q_hat_l  s_2 = s_m
        W_3 = q_r - q_hat_r      s_3 = max(u_r-c_r,u_r+c_r,lambda_roe_1,lambda_roe_2)
        s_m = (p_r - p_l + rho_l*u_l*(s_l - u_l) - rho_r*u_r*(s_r - u_r))\
          / (rho_l*(s_l-u_l) - rho_r*(s_r - u_r))

    left middle state::

        q_hat_l[0,:] = rho_l*(s_l - u_l)/(s_l - s_m)
        q_hat_l[1,:] = rho_l*(s_l - u_l)/(s_l - s_m)*s_m
        q_hat_l[2,:] = rho_l*(s_l - u_l)/(s_l - s_m)\
                *(E_l/rho_l + (s_m - u_l)*(s_m + p_l/(rho_l*(s_l - u_l))))

    right middle state::

        q_hat_r[0,:] = rho_r*(s_r - u_r)/(s_r - s_m)
        q_hat_r[1,:] = rho_r*(s_r - u_r)/(s_r - s_m)*s_m
        q_hat_r[2,:] = rho_r*(s_r - u_r)/(s_r - s_m)\
                *(E_r/rho_r + (s_m - u_r)*(s_m + p_r/(rho_r*(s_r - u_r))))

    *problem_data* should contain:

        - *gamma*: (float) Ratio of specific heat capacities
        - *gamma1*: (float) :math:`\gamma - 1`

    :Version 1.0 (2015-11-18)
    """

    # Problem dimensions
    num_rp = q_l.shape[1]
    num_waves = 3

    # Return values
    wave = np.empty( (num_eqn, num_waves, num_rp) )
    s = np.empty( (num_waves, num_rp) )
    amdq = np.zeros( (num_eqn, num_rp) )
    apdq = np.zeros( (num_eqn, num_rp) )
    
    # Solver parameters
    gamma1 = problem_data['gamma1']
    
    # Calculate Roe averages, right and left speeds
    u, a, _, p_l, p_r = roe_averages(q_l,q_r,problem_data)
    rho_r = q_r[0,:]
    rho_l = q_l[0,:]
    E_r = q_r[2,:]
    E_l = q_l[2,:]
    H_r = (E_r + p_r) / rho_r
    H_l = (E_l + p_l) / rho_l
    u_r = q_r[1,:] / rho_r
    u_l = q_l[1,:] / rho_l
    a_r = np.sqrt(gamma1 * (H_r - 0.5 * u_r**2))
    a_l = np.sqrt(gamma1 * (H_l - 0.5 * u_l**2))

    # Compute Einfeldt speeds
    s_index = np.empty((4,num_rp))
    s_index[0,:] = u + a
    s_index[1,:] = u - a
    s_index[2,:] = u_l + a_l
    s_index[3,:] = u_l - a_l
    s[0,:]  = np.min(s_index,axis=0)
    s_index[2,:] = u_r + a_r
    s_index[3,:] = u_r - a_r
    s[2,:] = np.max(s_index,axis=0)

    # left and right speeds
    s_l = s[0,:]
    s_r = s[2,:]
    
    # middle speed
    s_m = np.empty((num_rp))
    s_m[:] = (p_r - p_l + rho_l*u_l*(s_l - u_l) - rho_r*u_r*(s_r - u_r))\
          / (rho_l*(s_l-u_l) - rho_r*(s_r - u_r))
    s[1,:] = s_m
    
    # left middle states
    q_hat_l = np.empty((num_eqn,num_rp))
    q_hat_l[0,:] = rho_l*(s_l - u_l)/(s_l - s_m)
    q_hat_l[1,:] = rho_l*(s_l - u_l)/(s_l - s_m)*s_m
    q_hat_l[2,:] = rho_l*(s_l - u_l)/(s_l - s_m)\
                *(E_l/rho_l + (s_m - u_l)*(s_m + p_l/(rho_l*(s_l - u_l))))
    
    # right middle state
    q_hat_r = np.empty((num_eqn,num_rp))
    q_hat_r[0,:] = rho_r*(s_r - u_r)/(s_r - s_m)
    q_hat_r[1,:] = rho_r*(s_r - u_r)/(s_r - s_m)*s_m
    q_hat_r[2,:] = rho_r*(s_r - u_r)/(s_r - s_m)\
                *(E_r/rho_r + (s_m - u_r)*(s_m + p_r/(rho_r*(s_r - u_r))))

    # Compute each family of waves
    wave[:,0,:] = q_hat_l - q_l
    wave[:,1,:] = q_hat_r - q_hat_l
    wave[:,2,:] = q_r - q_hat_r
    
    # Compute variations
    s_index = np.zeros((2,num_rp))
    for m in range(num_eqn):
        for mw in range(num_waves):
            s_index[0,:] = s[mw,:]
            amdq[m,:] += np.min(s_index,axis=0) * wave[m,mw,:]
            apdq[m,:] += np.max(s_index,axis=0) * wave[m,mw,:]
            
    return wave, s, amdq, apdq

def euler_exact_1D(q_l,q_r,aux_l,aux_r,problem_data):
    r"""
    Exact euler Riemann solver
    
    .. warning::
        This solver has not been implemented.
    
    """
    raise NotImplementedError("The exact Riemann solver has not been implemented.")

def roe_averages(q_l,q_r,problem_data):
    # Solver parameters
    gamma1 = problem_data['gamma1']

    # Calculate Roe averages
    rhsqrtl = np.sqrt(q_l[0,...])
    rhsqrtr = np.sqrt(q_r[0,...])
    pl = gamma1 * (q_l[2,...] - 0.5 * (q_l[1,...]**2) / q_l[0,...])
    pr = gamma1 * (q_r[2,...] - 0.5 * (q_r[1,...]**2) / q_r[0,...])
    rhsq2 = rhsqrtl + rhsqrtr
    u = (q_l[1,...] / rhsqrtl + q_r[1,...] / rhsqrtr) / rhsq2
    enthalpy = ((q_l[2,...] + pl) / rhsqrtl + (q_r[2,...] + pr) / rhsqrtr) / rhsq2
    a = np.sqrt(gamma1 * (enthalpy - 0.5 * u**2))

    return u, a, enthalpy, pl, pr




#!/usr/bin/env python

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('fileio', parent_package, top_path)
    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python
# encoding: utf-8

"""Tests for reading and writing GeoClaw data files"""

import tempfile
import shutil
import os

import numpy

import clawpack.clawutil.test as test
import clawpack.geoclaw.data
import clawpack.geoclaw.fgmax_tools as fgmax_tools


def test_read_FGmaxData():
    r"""Test readinga and writing of FGmaxData files"""

    # Create temp directory
    temp_path = tempfile.mkdtemp()

    try:
        data_file = os.path.join(temp_path, "fgmax_grids.data")

        # Test data object
        fgmax_data = clawpack.geoclaw.data.FGmaxData()
        fgmax_data.num_fgmax_val = 2

        # Test grid data
        fg = fgmax_tools.FGmaxGrid()
        fg.point_style = 2
        fg.dx = 2.0 / (3.0 * 4.0)
        fg.x1 = -120.0 + fg.dx / 2.0
        fg.x2 = -60.0 - fg.dx / 2.0
        fg.y1 = -60.0 + fg.dx / 2.0
        fg.y2 = 0.0 - fg.dx / 2.0
        fg.tstart_max =  10.0
        fg.tend_max = 1.e10
        fg.dt_check = 60.0
        fg.min_level_check = 3
        fg.arrival_tol = 1.e-2
        fg.interp_method = 0
        fgmax_data.fgmax_grids.append(fg)
    
        fgmax_data.write(out_file=data_file)

        # Read data object
        read_fgmax_data = clawpack.geoclaw.data.FGmaxData()
        read_fgmax_data.read(data_file)

        # Tests
        tfg = read_fgmax_data.fgmax_grids[0]
        assert numpy.allclose(fg.x1, tfg.x1)
        assert numpy.allclose(fg.x2, tfg.x2)
        assert numpy.allclose(fg.y1, tfg.y1)
        assert numpy.allclose(fg.y2, tfg.y2)
        assert numpy.allclose(fg.tstart_max, tfg.tstart_max)
        assert numpy.allclose(fg.tend_max, tfg.tend_max)
        assert numpy.allclose(fg.dt_check, tfg.dt_check)
        assert numpy.allclose(fg.min_level_check, tfg.min_level_check)
        assert numpy.allclose(fg.arrival_tol, tfg.arrival_tol)
        assert numpy.allclose(fg.interp_method, tfg.interp_method)

    except AssertionError as e:
        # If the assertion failed then copy the contents of the directory
        shutil.copytree(temp_path, os.path.join(os.getcwd(), 
                                                "test_read_FGmaxData"))
        raise e

    finally:
        shutil.rmtree(temp_path)


if __name__ == "__main__":
    test_read_FGmaxData()
    print("All tests passed.")


from . import shock_bubble, shocktube, Sedov
import numpy as np
from pytest import importorskip
import os

thisdir = os.path.dirname(__file__)
path_expected_sols = os.path.join(thisdir,'expected_sols.npy')
expected_sols_dict = np.load(path_expected_sols,allow_pickle=True).item()

def error(test_name,**kwargs):
    if "shocktube" in test_name:
        claw = shocktube.shocktube(**kwargs)
    elif "shock_bubble" in test_name:
        claw = shock_bubble.setup(**kwargs)
    else:
        raise ValueError(f"Test name {test_name} not recognized")
    claw.run()
    density_test = claw.frames[claw.num_output_times].state.get_q_global()[0].reshape([-1])
    assert test_name in expected_sols_dict.keys(), f"Test name {test_name} not found in {path_expected_sols}"
    density_expected = expected_sols_dict[test_name]
    dx,dy,dz = claw.solution.domain.grid.delta
    diff_L1 = dx*dy*dz*np.sum(np.abs(density_test-density_expected))
    return diff_L1

class TestEuler3D:
    def test_shocktube(self):
        assert error(test_name='shocktube',mx=10,my=10,mz=128, tfinal=0.2,disable_output=True)<1e-6
    
    def test_shock_bubble(self):
        _ = importorskip("scipy") # skip test if scipy is not installed
        assert error(test_name='shock_bubble',num_cells=(64,16,16),tfinal=0.1,disable_output=True)<1e-6

    def test_sedov_and_hdf(self):
        _ = importorskip("scipy") # skip test if scipy is not installed
        _ = importorskip("h5py") # skip test if h5py is not installed
        from clawpack.pyclaw import Solution

        # Read expected solution
        sol_expected = Solution()
        sol_expected.read(1,path=os.path.join(thisdir,'Sedov_regression'),
                          file_format='hdf',read_aux=False, file_prefix='claw')
        density_expected = sol_expected.state.get_q_global()[0].reshape([-1])

        #Run test
        tempdir = os.path.join(thisdir,'_sedov_test_results')
        claw = Sedov.setup(solver_type='classic',
                                 outdir=tempdir, num_cells=(16, 16, 16),
                                 num_output_times=1, tfinal=0.1)
        claw.run()

        # Read test solution
        sol_test = Solution()
        sol_test.read(1,path=tempdir,file_format='hdf',read_aux=False,file_prefix='claw')
        density_test = sol_test.state.get_q_global()[0].reshape([-1])
        dx,dy,dz = sol_expected.domain.grid.delta
        diff_L1 = dx*dy*dz*np.sum(np.abs(density_test-density_expected))
        assert diff_L1<1e-6
    



"""
Make piecewise linear topography for wave tank.
"""

from pylab import *
from clawpack.geoclaw import nonuniform_grid_tools

xlower = -15. #-0.98
xupper = 8.19

xzpairs = [(-0.98, -0.218),       # left edge
           (    0, -0.218),       # start of first slope
           ( 4.36, -0.1357),      # start of second slope
           ( 7.29, -0.1162),      # start of third slope
           ( 8.19, -0.0470)]      # right edge

# flat:
#xzpairs = [(xlower, -0.218),       # left edge
#           (xupper, -0.218)]       # right edge

topo_fcn = nonuniform_grid_tools.make_pwlin_topo_fcn(xzpairs)

mx = 800
hmin = 0.04  # use uniform grid in shallower water

nonuniform_grid_tools.make_celledges_cfl(xlower, xupper, mx, topo_fcn,
        hmin, fname='celledges.data', plot_topo=True)




#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import print_function
import sys

try:
    import numpy
except:
    sys.path.append("/opt/share/ksl/numpy/dev-aug29/ppc450d/lib/python/")
    import numpy

import ksl_rescale 

a=numpy.array([[1,2],[3,4]],dtype=float,order='FORTRAN')
print(a)
print("rescaling by 2!")
ksl_rescale.rescale(a,2.0)
print(a)



#!/usr/bin/env python
# encoding: utf-8
from __future__ import absolute_import
IC='gauss_square'
if IC=='gauss_square':
    beta=200.; x0=0.3; mx=100
elif IC=='wavepacket':
    beta=100.; x0=0.5; mx=100

def fig_61_62_63(solver_order=2, limiters=0):
    """
    Compare several methods for advecting a Gaussian and square wave.

    The settings coded here are for Figure 6.1(a).
    For Figure 6.1(b), set solver.order=2.
    For Figure 6.2(a), set solver.order=2 and solver.limiters = pyclaw.limiters.tvd.minmod (1)
    For Figure 6.2(b), set solver.order=2 and solver.limiters = pyclaw.limiters.tvd.superbee (2)
    For Figure 6.2(c), set solver.order=2 and solver.limiters = pyclaw.limiters.tvd.MC (4)

    For Figure 6.3, set IC='wavepacket' and other options as appropriate.
    """
    import numpy as np
    from clawpack import pyclaw
    from clawpack import riemann

    solver = pyclaw.ClawSolver1D(riemann.advection_1D)

    solver.bc_lower[0] = 2
    solver.bc_upper[0] = 2
    solver.limiters = limiters
    solver.order = solver_order
    solver.cfl_desired = 0.8

    x = pyclaw.Dimension(0.0,1.0,mx,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 1
    state = pyclaw.State(domain, num_eqn)
    state.problem_data['u'] = 1.

    xc = domain.grid.x.centers
    if IC=='gauss_square':
        state.q[0,:] = np.exp(-beta * (xc-x0)**2) + (xc>0.6)*(xc<0.8)
    elif IC=='wavepacket':
        state.q[0,:] = np.exp(-beta * (xc-x0)**2) * np.sin(80.*xc)
    else:
        raise Exception('Unrecognized initial condition specification.')

    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver

    claw.tfinal = 10.0
    return claw


if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(fig_61_62_63)



"""Main pyclaw package"""

import os
import logging, logging.config

_init = os.path.abspath(__file__)
_root = os.path.dirname(os.path.dirname(os.path.dirname(_init)))
if os.path.isdir(_root):
    __path__.append(_root)

# Default logging configuration file
_DEFAULT_LOG_CONFIG_PATH = os.path.join(os.path.dirname(__file__),'log.config')
del os, _init, _root

# Setup loggers
logging.config.fileConfig(_DEFAULT_LOG_CONFIG_PATH, disable_existing_loggers=False)

__all__ = []

# Module imports
__all__.extend(['Controller','Dimension','Patch','Domain','Solution','State','CFL','plot'])
from .controller import Controller
from .solution import Solution
from .geometry import Dimension, Patch, Domain
from .state import State
from .cfl import CFL

from clawpack.pyclaw import plot

__all__.extend(['ClawSolver1D','ClawSolver2D','ClawSolver3D','SharpClawSolver1D','SharpClawSolver2D','SharpClawSolver3D'])
from .classic.solver import ClawSolver1D, ClawSolver2D, ClawSolver3D
from .sharpclaw.solver import SharpClawSolver1D, SharpClawSolver2D,SharpClawSolver2D,SharpClawSolver3D


# Sub-packages
from . import limiters
from .limiters import *
__all__.extend(limiters.__all__)

__all__.append('BC')
from .solver import BC
__all__.extend('IOTest')
from .tests.test_io import IOTest



#!/usr/bin/env python
# encoding: utf-8
r"""
Routines for reading and writing an ascii output file from ForestClaw
"""

from __future__ import absolute_import
from __future__ import print_function

import logging
import numpy as np
# import clawpack.forestclaw as forestclaw
from clawpack import forestclaw

from .. import Dimension, Patch
from clawpack.pyclaw.fileio.ascii import write
from clawpack.pyclaw.fileio.ascii import read
from clawpack.pyclaw.util import read_data_line



logger = logging.getLogger('pyclaw.fileio')


def write_patch_header(f, patch):
    f.write("%5i                  patch_number\n" % patch.patch_index)
    f.write("%5i                  AMR_level\n" % patch.level)
    f.write("%5i                  block_number\n" % patch.block_number)
    f.write("%5i                  mpi_rank\n" % patch.mpi_rank)
    for dim in patch.dimensions:
        f.write("%5i                  m%s\n" % (dim.num_cells, dim.name))
    for dim in patch.dimensions:
        f.write("%18.8e     %slow\n" % (dim.lower, dim.name))
    for dim in patch.dimensions:
        f.write("%18.8e     d%s\n" % (dim.delta, dim.name))

    f.write("\n")


def read_patch_header(f, num_dim):
    r"""Read header describing the next patch

    :Input:
     - *f* - (file) Handle to open file
     - *num_dim* - (int) Number of dimensions

    :Output:
     - *patch* - (clawpack.pyclaw.geometry.Patch) Initialized patch represented
       by the header data.
    """

    n = np.zeros((num_dim))
    d = np.zeros((num_dim))
    lower = np.zeros((num_dim))

    patch_index = read_data_line(f, data_type=int)
    level = read_data_line(f, data_type=int)
    block_number = read_data_line(f, data_type=int)
    mpi_rank = read_data_line(f, data_type=int)
    for i in range(num_dim):
        n[i] = read_data_line(f, data_type=int)
    for i in range(num_dim):
        lower[i] = read_data_line(f)
    for i in range(num_dim):
        d[i] = read_data_line(f)

    blank = f.readline()

    # Construct the patch
    # Since we do not have names here, we will construct the patch with
    # dimension names x,y,z
    names = ['x', 'y', 'z']
    dimensions = [forestclaw.geometry.Dimension(lower[i], lower[i] + n[i] * d[i],
                  n[i], name=names[i]) for i in range(num_dim)]
    patch = forestclaw.geometry.Patch(dimensions)

    # Add AMR attributes:
    patch.patch_index = patch_index
    patch.level = level

    # Add ForestClaw attributes
    patch.block_number = block_number
    patch.mpi_rank = mpi_rank

    return patch

# # Replace the ascii module functions with those defined above
# # ascii.read_patch_header = read_patch_header
# # ascii.write_patch_header = write_patch_header
# read = ascii.read
# write = ascii.write



"Runs the shocktube test problem with the Fortran Roe solver."

def test_shocktube():
    "Shock tube test (1D MHD with Roe solver)"
    import numpy as np
    import os
    from . import shocktube
    from clawpack.riemann.mhd_1D_constants import B_2, B_3

    claw = shocktube.setup(kernel_language='Fortran')
    claw.run()
    roe = claw.solution.state.get_q_global()

    if roe is not None:
        thisdir = os.path.dirname(__file__)
        expected_B2 = np.loadtxt(os.path.join(thisdir, 'shocktube_regression_B2.txt'))
        expected_B3 = np.loadtxt(os.path.join(thisdir, 'shocktube_regression_B3.txt'))
        test_B2 = roe[B_2, :]
        test_B3 = roe[B_3, :]
        test_err = np.linalg.norm(expected_B2 - test_B2) + np.linalg.norm(expected_B3 - test_B3)
        assert test_err < 1.e-4




import ctypes
import os.path

class iso_c_rp1_advection():

    def __init__(self, u):
        this_path = os.path.dirname(__file__)
        self._dll = ctypes.CDLL(os.path.join(this_path, '_iso_c_advection.so'))
        self.context = ctypes.c_void_p()
        self._u = ctypes.c_double(u)
        self._dll.rp1_advection_new(ctypes.byref(self._u),
                                   ctypes.byref(self.context))
        self.num_eqn = 1
        self.num_waves = 1
        self._rp = self._dll.rp1_advection_c

    def __del__(self):
        self._dll.rp1_advection_delete(ctypes.byref(self.context))



def test_shocksine():
    """ Test shock-sine wave interaction (Euler 1D)"""
    from . import shocksine
    from clawpack.pyclaw.util import test_app, check_diff
    import os

    thisdir = os.path.dirname(__file__)
    
    def verify_shocksine(controller):
        """ given an expected value, returns a verification function """
        import numpy as np

        test_solution = controller.solution.state.get_q_global()

        if test_solution is not None:
            expected_density = np.loadtxt(os.path.join(thisdir,'shocksine_regression_density.txt'))
            test_density = test_solution[0,:]
            test_err = np.linalg.norm(expected_density-test_density)
            return check_diff(0, test_err, abstol=1.e-4)

    return test_app(shocksine.setup, verify_shocksine, {'outdir': os.path.join(thisdir,'./_output')})






"""
Generic code for plotting Clawpack results.

Execute from unix by
    $ python $CLAW/visclaw/src/python/visclaw/plotclaw.py
from a directory that contains a file setplot.py that sets other
parameters to specify the desired plots, location of data, etc.

If a different file is to be used to define the function setplot, this can
be given as an argument, e.g.
    $ python $CLAW/visclaw/src/python/visclaw/plotclaw.py setplot_alternative.py

From most Clawpack applications directories the command
    $ make .plots
will call the plotclaw function from this module.

"""

import matplotlib
matplotlib.use('Agg') 

import sys
import os
import time
import subprocess

import clawpack.visclaw.frametools as frametools

if sys.platform in ['win32','cygwin']:
    pypath = 'C:/cygwin' + os.environ['CLAW'] + '/python'
    sys.path.append(pypath)


def plotclaw(outdir='.', plotdir='_plots', setplot = 'setplot.py',
             format='ascii', msgfile='', frames=None, verbose=False):
    """
    Create html and/or latex versions of plots.

    INPUT:
        setplot is a module containing a function setplot that will be called
                to set various plotting parameters.
        format specifies the format of the files output from Clawpack
    """

    from clawpack.visclaw.data import ClawPlotData
    from clawpack.visclaw import plotpages

    plotdata = ClawPlotData()
    plotdata.outdir = outdir
    plotdata.plotdir = plotdir
    plotdata.setplot = setplot
    plotdata.format = format
    plotdata.msgfile = msgfile


    frametools.call_setplot(plotdata.setplot, plotdata)


    if plotdata.num_procs is None:
        plotdata.num_procs = int(os.environ.get("OMP_NUM_THREADS", 1))

    # Make sure plotdata.parallel is False in some cases:


    if plotdata.parallel:
        assert type(setplot) in [str, bool, type(None)], \
                "*** Parallel plotting is not supported when ClawPlotData " \
                + "attribute setplot is a function."

    if plotdata.parallel and (plotdata.num_procs > 1):

        # If this is the original call then we need to split up the work and 
        # call this function again

        # First set up plotdir:

        plotdata._parallel_todo = 'initialize'
        plotpages.plotclaw_driver(plotdata, verbose=False, format=format)

        if frames is None:
            if plotdata.num_procs is None:
                plotdata.num_procs = int(os.environ.get("OMP_NUM_THREADS", 1))


            frames = [[] for n in range(plotdata.num_procs)]
            framenos = frametools.only_most_recent(plotdata.print_framenos,
                                                   outdir)

            # don't use more procs than frames or infinite loop!!
            num_procs = min(plotdata.num_procs, len(framenos))

            for (n, frame) in enumerate(framenos):
                frames[n%num_procs].append(frame)

            # Create subprocesses to work on each
            plotclaw_cmd = "python %s" % __file__
            process_queue = []
            for n in range(num_procs):
                plot_cmd = "%s %s %s %s" % (plotclaw_cmd, 
                                            outdir,
                                            plotdir, 
                                            setplot)
                plot_cmd = plot_cmd + " " + " ".join([str(i) for i in frames[n]])

                process_queue.append(subprocess.Popen(plot_cmd, shell=True))

            poll_interval = 5
            try:
                while len(process_queue) > 0:
                    time.sleep(poll_interval)
                    for process in process_queue:
                        if process.poll() is not None:
                            process_queue.remove(process)
                    if verbose:
                        print("Number of processes currently:",len(process_queue))
            
            # Stop child processes if interrupt was caught or something went 
            # wrong
            except KeyboardInterrupt:
                print("ABORTING: A keyboard interrupt was caught.  All " + \
                      "child processes will be terminated as well.")
                for process in process_queue:
                    process.terminate()
                raise

            except:
                print("ERROR: An error occurred while waiting for " + \
                      "plotting processes to complete.  Aborting all " + \
                      "child processes.")
                for process in process_queue:
                    process.terminate()
                raise 

            # After all frames have been plotted via recursive calls,
            # make index and gauge plots only:
            plotdata._parallel_todo = 'finalize'
            plotpages.plotclaw_driver(plotdata, verbose=False, format=format)

        else:
            # make frame plots only:
            plotdata._parallel_todo = 'frames'
            plotdata.print_framenos = frames
            plotpages.plotclaw_driver(plotdata, verbose=False, format=format)

    else:
        # not in parallel:
        plotdata._parallel_todo = None
        plotpages.plotclaw_driver(plotdata, verbose=False, format=format)



if __name__ == '__main__':
    """
    If executed at command line prompt, simply call the function, with
    any arguments passed in.
    """

    if len(sys.argv) > 4:
        frames = [int(frame) for frame in sys.argv[4:]]
        plotclaw(sys.argv[1], sys.argv[2], sys.argv[3], frames=frames)
    elif len(sys.argv) == 4:
        plotclaw(sys.argv[1], sys.argv[2], sys.argv[3])
    elif len(sys.argv) == 3:
        plotclaw(sys.argv[1], sys.argv[2])
    elif len(sys.argv) == 2:
        plotclaw(sys.argv[1])
    else:
        plotclaw()
